//    Input Output management   functions of RDS USB software 
//
// =====================================================
//  Description
//    This file include initialisation and management functions
//       of IO mgt 
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// History:
// 29/05/07    V1.0
//       creation 
//=======================================


#include "RDS_IOMgt.h"
#include "RDS_ClockDriver.h"
#include "RDS.h"
#include "RDS_RS232.h"  /* for debug functionnalities */
#include "RDS_EepromMgt.h"


/* include system */
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>   
#include <avr/interrupt.h>

/* local constant */

/* static variable  */

/* initialisation */
void RDSIO_Init(void)
{


  /* active pull up resistor for TA input */
   RDS_TA_IN_REG |=  (1<<RDS_TA_IN_BIT);   /* set to 1 */
  
  /* configure  output for TALED  output */
  RDS_TALED_OUT_DIR |= (1<<RDS_TALED_OUT_BIT);
  /* configure  output for activity LED  output */
  RDS_ACTLED_OUT_DIR |= (1<<RDS_ACTLED_OUT_BIT);
  /* first setting = LED ON */ 
  RDS_TALED_OUT_REG &= ~(1<<RDS_TALED_OUT_BIT);
  RDS_ACTLED_OUT_REG &= ~(1<<RDS_ACTLED_OUT_BIT);

  /* configure output for 57K External clock select */ 
  RDS_57KEXT_OUT_DIR |= (1<<RDS_57KEXT_OUT_BIT);
  /* configure output for 57K Internal clock select */ 
  RDS_57KINT_OUT_DIR |= (1<<RDS_57KINT_OUT_BIT);

  if (EEP_get_StereoMode() != 0)
  {  
     /* configure for stereo mode */
     RDS_57KINT_OUT_REG &= ~(1<<RDS_57KINT_OUT_BIT);   /* set to 0 */
     RDS_57KEXT_OUT_REG |=  (1<<RDS_57KEXT_OUT_BIT);   /* set to 1 */

  }
  else
  {
      /* configure for mono mode  */
      RDS_57KINT_OUT_REG |=  (1<<RDS_57KINT_OUT_BIT);   /* set to 1 */
      RDS_57KEXT_OUT_REG &= ~(1<<RDS_57KEXT_OUT_BIT);   /* set to 0 */  


  }

  /* configure output for RDS data output */
  RDS_DATA_OUT_DIR |= (1<<RDS_DATA_OUT_BIT);
  
  /* configure OC1A as output for timer 1 */
  RDS_OC1A_OUT_DIR |= (1<<RDS_OC1A_OUT_BIT);
  


}

/* set TA LED 0: OFF, 1 LED ON */                            
void RDSIO_SetTALed(unsigned char NewState)
{
   if (NewState !=0)
   {
      RDS_TALED_OUT_REG &= ~(1<<RDS_TALED_OUT_BIT);
   }
   else
   {
      RDS_TALED_OUT_REG |= (1<<RDS_TALED_OUT_BIT);
   }
}                 

/* change current state of TA LED */
void RDSIO_ToggleTALed(void)
{
     /* toggle TA LED */
     RDS_TALED_OUT_REG ^= (1<<RDS_TALED_OUT_BIT);
}                     

/* set Activity LED 0: OFF, 1 LED ON */
void RDSIO_SetActLed(unsigned char NewState)
{
   if (NewState !=0)
   {
      RDS_ACTLED_OUT_REG &= ~(1<<RDS_ACTLED_OUT_BIT);
   }
   else
   {
      RDS_ACTLED_OUT_REG |= (1<<RDS_ACTLED_OUT_BIT);
   }
}            

/* change current state of Activity LED */
void RDSIO_ToggleActLed(void)
{
  /* toggle ACT LED  */
  RDS_ACTLED_OUT_REG ^= (1<<RDS_ACTLED_OUT_BIT);
}

/* read TA input return 1  if active level */
unsigned char RDSIO_ReadTAInput(void)
{
   if ((RDS_TA_IN_REG_IN &(1<<RDS_TA_IN_BIT))!=0)
   {
    return (0);
   }
   else
   {
    return (1);    
   }
}             







