// header file for RS232 driver of RDS software 
//
// =====================================================
//  Decription
//    This file include constant definition and prototype  functions of RS232
//    
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
//=======================================
// Author : G.Laroche www.g.laroche.free.fr
//=======================================
// Historique:
// 19/05/07    V1.0
//       Create
//=======================================


#ifndef RDS_RS232_H
#define RDS_RS232_H

/* Includes */
#include <avr/io.h>      /* ATmega IO header */
#include <avr/signal.h>  /* interrupt header */
#include <avr/pgmspace.h>


/* UART Buffer Defines */
#define RS_RX_BUFFER_SIZE 128     /* 2,4,8,16,32,64,128 or 256 bytes */
#define RS_TX_BUFFER_SIZE 128     /* 2,4,8,16,32,64,128 or 256 bytes */
#define RS_RX_BUFFER_MASK ( RS_RX_BUFFER_SIZE - 1 )
#define RS_TX_BUFFER_MASK ( RS_TX_BUFFER_SIZE - 1 )
#if ( RS_RX_BUFFER_SIZE & RS_RX_BUFFER_MASK )
	#error RX buffer size is not a power of 2
#endif
#if ( RS_TX_BUFFER_SIZE & RS_TX_BUFFER_MASK )
	#error TX buffer size is not a power of 2
#endif


 
/* Prototypes functions */

void RS_Init( unsigned int baudrate );     // RS initialise 
/* low level functions */
unsigned char RS_Receive( void );          // RS receive
void RS_Transmit( unsigned char data );    // RS send
unsigned char DataInReceiveBuffer( void ); // return 0 if no data in receive buffer

/* application level function    */
/*-------------------------------*/
/* command buffer management  */
unsigned char RS_ReadCmdChar(void);         // return the next character including command number management 
unsigned int  RS_ReadCmdParamUi(void);      // read a parameter Unsigned Int and return it after
void RS_ReadCmdParamStr(char * ParamReadOut,unsigned char MaxRead);     // read a string delimited by  ' ' or '\r' and return it,  max size :MaxRead
void RS_decNbCmd(void);                      // decrementing command number 
unsigned char RS_getNbCmd(void);             // return the number of command in buffer receive 

void RS_ReceiveMgt( void );                 /* receive command management*/


/* Char sending management functions using Interrupt and sending buffer  */
void RS_PrintfStrPGM( PGM_P data );            // Send a string from pgm space (flash)
void RS_PrintfStr( unsigned char  * data );    // Send a string from ram space 
void RS_Printfc( unsigned char  Caractere );   // send a single char 
void RS_PrintfUi( unsigned int  Data, unsigned char Format ); // send a unsigned char in decimal (if Format=0) or hexa (if Format=1)

/* Char sending management functions for debug without using Interrupt, all Debug functions wait for free register before sending data   */
void RS_DebugPrintfc( unsigned char  Caractere );   // polling send of a single char 
void RS_DebugPrintfStr( unsigned char  * StringChar );   // polling send a string from ram space 
void RS_DebugPrintfUi( unsigned int  Data, unsigned char Format ); // polling send a unsigned char in decimal (if Format=0) or hexa (if Format=1)
void RS_DebugPrintfStrPGM( PGM_P data );            // polling send a string from pgm space (flash)

//void RS_DebugDumpSRAM(void);                   //  dump SRAM to check stack limit 

 


#endif
